@@ -1,8 +1,10 @@
 use strict;
 package Event::process;
+use base 'Event::Watcher';
 use vars qw($DefaultPriority);
 $DefaultPriority = Event::PRIO_HIGH();
 
+BEGIN { 'Event::Watcher'->import(qw(ACTIVE)); }
 'Event::Watcher'->register();
 
 sub new {
@@ -42,12 +44,14 @@ Event->signal(signal => 'CHLD',  #CLD? XXX
 
 sub start {
     my ($o, $repeat) = @_;
+    $o->{flags} |= ACTIVE;
     my $key = exists $o->{any}? 'any' : $o->{pid};
     push @{$cb{ $key } ||= []}, $o;
 }
 
 sub stop {
     my $o = shift;
+    $o->{flags} &= ~ACTIVE;
     my $key = exists $o->{any}? 'any' : $o->{pid};
     $cb{ $key } = [grep { $_ != $o } @{$cb{ $key }} ];
     delete $cb{ $key } if
